==> corekeyboard-2.7.0.tar.gz: corekeyboard-v2.7.0/ui/button.cpp <==
/*
	*
	* This file is a part of CoreKeyboard.
	* An on-screenkeyboard for CuboCore Application Suite
	* Copyright 2019 Britanicus <marcusbritanicus@gmail.com>
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, see {http://www.gnu.org/licenses/}.
	*
*/


#include "button.h"
#include "settings.h"

#include <QWidget>
#include <QTimer>
#include <QPainter>
#include <QX11Info>
#include <QHBoxLayout>

#include <math.h>

#include <X11/XKBlib.h>
#include "fixx11h.h"

#define XK_Caps_Lock 0xffe5

static QList<int> modifiers = QList<int>() << 50 << 62 << 64 << 108 << 37 << 109 << 133;
static QList<int> functions = QList<int>() << 9 << 67 << 68 << 69 << 70 << 71 << 72 << 73 << 74 << 75 << 76 << 95 << 96 << 118 << 119;
static QList<int> longkeys = QList<int>() << 22 << 23 << 66 << 104 << 50 << 62 << 65;

const static QString alphabet = "abcdefghijklmnopqrstuvwxyz";

bool isBrightColor( QColor dst, QColor src ) {

	if ( abs( dst.lightness() - src.lightness() ) > 64 )
		return true;

	else
		return false;
};

Button::Button( unsigned int code, QString name, QString altName, QWidget *parent ) : QWidget( parent ) {

	/* Flags and state init */
	mChecked = false;
	mCheckable = false;
	mPressed = false;
	mStretch = 1.0;
	mPersist = false;

	mName = name;
	mAltName = altName;

	text = mName;

	QPalette pltt = palette();
	mColor = QColor( Qt::black );

	/* X11 KeyCode */
	mKeyCode = code;

	if ( modifiers.contains( code ) )
		mCheckable = true;

	if ( CKSettings::keyboardLayout() == CKSettings::Layout::Mobile ) {
		if ( code == 65 )
			mStretch = 2;

		else if ( ( code == 50 ) or ( code == 62 ) or ( code == 22 )  )
			mStretch = 1.5;
	}

	else {
		if ( longkeys.contains( code ) ) {
			/* Stretch to maximum available space */
			QHBoxLayout *lyt = new QHBoxLayout();
			lyt->addStretch();
			setLayout( lyt );
		}
	}

	/* Check if we have a bad contrast */
	if ( not isBrightColor( pltt.color( QPalette::Window ), mColor ) ) {
		/* If we have a dark BG, lighten the colors */
		if ( pltt.color( QPalette::Window ).lightness() < 128 )
			mColor = mColor.lighter();

		/* If we have a light background, darken the colors */
		else
			mColor = mColor.darker();
	}

	if ( ( mColor.name() == "#000000" ) and pltt.color( QPalette::Window ).lightness() < 128 )
		mColor = QColor( 180, 180, 180 );

	/* Enable mouse tracking */
	setMouseTracking( true );

	if ( mKeyCode == 22 ) {
		longPressTimer = new QTimer( this );
		longPressTimer->setSingleShot( false );
		longPressTimer->setInterval( 200 );

		connect( longPressTimer, SIGNAL( timeout() ), this, SLOT( repeatBackSpace() ) );
	}

	else {
		longPressTimer = new QTimer( this );
		longPressTimer->setSingleShot( true );
		longPressTimer->setInterval( 300 );

		connect( longPressTimer, SIGNAL( timeout() ), this, SLOT( autoReleaseKey() ) );
	}
}

unsigned int Button::keyCode() {

	return mKeyCode;
};

bool Button::persist() {

    return mPersist;
};

bool Button::isChecked() {

    return mChecked;
};

void Button::setChecked( bool state ) {

    mChecked = state;
    emit clicked();

    repaint();
};

void Button::resize( int width, int height ) {

    setFixedHeight( height );
    setMinimumWidth( width * mStretch );
};

void Button::changeCase() {

	/* If the button name is longer than 1 char skip changing the case */
	if ( mName.count() > 1 )
		return;

	/* If button name is empty or, text is empty skip it */
	if ( not mName.count() or not text.count() )
		return;

	/* If there is an alternate name for the button, switch to it */
	if ( mAltName.count() ) {
		if ( mAltName == text )
			text = mName;

		else
			text = mAltName;
	}

	else {
		/* Skip if the text is not an alphabet */
		if ( not alphabet.contains( text.toLower() ) )
			return;

		if ( text.at( 0 ).isUpper() )
			text = text.toLower();

		else
			text = text.toUpper();
	}

	repaint();
};

void Button::autoReleaseKey() {

	if ( rect().contains( mapFromGlobal( QCursor::pos() ) ) ) {
		emit longPressed();

		/* Persist 'Shift' key */
		if ( ( mKeyCode == 50 ) or ( mKeyCode == 50 ) ) {
			mPersist = true;
			mChecked = true;
		}
	}

	mPressed = false;

	/* Stop long-press timer */
	longPressTimer->stop();

	repaint();
};

void Button::repeatBackSpace() {

	/* BackSpace key is 22 */
	if ( rect().contains( mapFromGlobal( QCursor::pos() ) ) ) {
		emit sendKey( 3722 );
	}
};

void Button::mousePressEvent( QMouseEvent *mEvent ) {

	mPressed = true;
	longPressTimer->start();

	mEvent->accept();

	repaint();
};

void Button::mouseReleaseEvent( QMouseEvent *mEvent ) {

	/* To enable long-press keys */
	if ( not mPressed )
		return;

	/* Unset the pressed flag */
	mPressed = false;

	/* Disable persistance */
	mPersist = false;

	/* Stop long-press timer */
	longPressTimer->stop();

    if ( mCheckable ) {
		mChecked = not mChecked;
		emit clicked();
	}

    else if ( rect().contains( mEvent->pos() ) ) {
        emit clicked();
        emit sendKey( keyCode() );
    }

    mEvent->accept();

    repaint();
};

void Button::paintEvent( QPaintEvent *pEvent ) {

	if ( not isEnabled() )
		return;

	QPainter *painter = new QPainter( this );
	painter->setRenderHints( QPainter::Antialiasing );

	painter->fillRect( rect(), Qt::transparent );

	painter->save();

	painter->setPen( QColor( 0, 0, 0, 150 ) );
	painter->setBrush( QColor( 0, 0, 0, 50 ) );

	/* If we want to hide key borders */
	if ( not CKSettings::showKeyBorders() ) {
		if ( mKeyCode != 65 ) {
			painter->setPen( Qt::NoPen );
			painter->setBrush( Qt::NoBrush );
		}
	}

	if ( mPressed or mChecked ) {
		QColor bColor = palette().color( QPalette::Text );
		bColor.setAlphaF( 0.3 );
		painter->setBrush( bColor );
	}

	// painter->drawRoundedRect( rect().adjusted( 0, 0, -1, -1 ), 3, 3 );
	painter->drawRect( rect().adjusted( 0, 0, -1, -1 ) );

	/* If we have a persisting button down, underline it */
	if ( mPersist ) {
		QColor bColor( Qt::darkCyan );
		bColor.setAlphaF( 0.5 );
		painter->setPen( QPen( bColor, 2 ) );
		painter->drawRoundedRect( rect().adjusted( 1, 1, -2, -2 ), 3, 3 );
	}

	painter->restore();

	painter->save();

	QFont btnFont( font() );
	btnFont.setPixelSize( height() * 0.5 );
	painter->setFont( btnFont );

	if ( mName.startsWith( ":/resources/" ) ) {
		qreal h = height() * 0.8;
		painter->drawPixmap( QPointF( ( width() - h ) / 2, ( height() - h ) / 2 ), QPixmap( mName ).scaled( h, h, Qt::KeepAspectRatio, Qt::SmoothTransformation ) );
	}

	else {
		QRect rect = QRect( QPoint( 0, 0 ), size() );
		painter->drawText( rect, Qt::AlignCenter, text );
	}

	painter->restore();
	painter->end();

	QWidget::paintEvent( pEvent );
};

==> corekeyboard-2.8.0.tar.gz: corekeyboard-v2.8.0/ui/button.cpp <==
/*
	*
	* This file is a part of CoreKeyboard.
	* An on-screenkeyboard for CuboCore Application Suite
    * Copyright 2019 CuboCore Group
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, see {http://www.gnu.org/licenses/}.
	*
*/


#include "button.h"
#include "settings.h"

#include <QWidget>
#include <QTimer>
#include <QPainter>
#include <QX11Info>
#include <QHBoxLayout>

#include <math.h>

#include <X11/XKBlib.h>
#include "fixx11h.h"

#define XK_Caps_Lock 0xffe5

static QList<int> modifiers = QList<int>() << 50 << 62 << 64 << 108 << 37 << 109 << 133;
static QList<int> functions = QList<int>() << 9 << 67 << 68 << 69 << 70 << 71 << 72 << 73 << 74 << 75 << 76 << 95 << 96 << 118 << 119;
static QList<int> longkeys = QList<int>() << 22 << 23 << 66 << 104 << 50 << 62 << 65;

const static QString alphabet = "abcdefghijklmnopqrstuvwxyz";

bool isBrightColor( QColor dst, QColor src ) {

	if ( abs( dst.lightness() - src.lightness() ) > 64 )
		return true;

	else
		return false;
};

Button::Button( unsigned int code, QString name, QString altName, QWidget *parent ) : QWidget( parent ) {

	/* Flags and state init */
	mChecked = false;
	mCheckable = false;
	mPressed = false;
	mStretch = 1.0;
	mPersist = false;

	mName = name;
	mAltName = altName;

	text = mName;

	QPalette pltt = palette();
	mColor = QColor( Qt::black );

	/* X11 KeyCode */
	mKeyCode = code;

	if ( modifiers.contains( code ) )
		mCheckable = true;

	if ( CKSettings::keyboardLayout() == CKSettings::Layout::Mobile ) {
		if ( code == 65 )
			mStretch = 2;

		else if ( ( code == 50 ) or ( code == 62 ) or ( code == 22 )  )
			mStretch = 1.5;
	}

	else {
		if ( longkeys.contains( code ) ) {
			/* Stretch to maximum available space */
			QHBoxLayout *lyt = new QHBoxLayout();
			lyt->addStretch();
			setLayout( lyt );
		}
	}

	/* Check if we have a bad contrast */
	if ( not isBrightColor( pltt.color( QPalette::Window ), mColor ) ) {
		/* If we have a dark BG, lighten the colors */
		if ( pltt.color( QPalette::Window ).lightness() < 128 )
			mColor = mColor.lighter();

		/* If we have a light background, darken the colors */
		else
			mColor = mColor.darker();
	}

	if ( ( mColor.name() == "#000000" ) and pltt.color( QPalette::Window ).lightness() < 128 )
		mColor = QColor( 180, 180, 180 );

	/* Enable mouse tracking */
	setMouseTracking( true );

	if ( mKeyCode == 22 ) {
		longPressTimer = new QTimer( this );
		longPressTimer->setSingleShot( false );
		longPressTimer->setInterval( 200 );

		connect( longPressTimer, SIGNAL( timeout() ), this, SLOT( repeatBackSpace() ) );
	}

	else {
		longPressTimer = new QTimer( this );
		longPressTimer->setSingleShot( true );
		longPressTimer->setInterval( 300 );

		connect( longPressTimer, SIGNAL( timeout() ), this, SLOT( autoReleaseKey() ) );
	}
}

unsigned int Button::keyCode() {

	return mKeyCode;
};

bool Button::persist() {

    return mPersist;
};

bool Button::isChecked() {

    return mChecked;
};

void Button::setChecked( bool state ) {

    mChecked = state;
    emit clicked();

    repaint();
};

void Button::resize( int width, int height ) {

    setFixedHeight( height );
    setMinimumWidth( width * mStretch );
};

void Button::changeCase() {

	/* If the button name is longer than 1 char skip changing the case */
	if ( mName.count() > 1 )
		return;

	/* If button name is empty or, text is empty skip it */
	if ( not mName.count() or not text.count() )
		return;

	/* If there is an alternate name for the button, switch to it */
	if ( mAltName.count() ) {
		if ( mAltName == text )
			text = mName;

		else
			text = mAltName;
	}

	else {
		/* Skip if the text is not an alphabet */
		if ( not alphabet.contains( text.toLower() ) )
			return;

		if ( text.at( 0 ).isUpper() )
			text = text.toLower();

		else
			text = text.toUpper();
	}

	repaint();
};

void Button::autoReleaseKey() {

	if ( rect().contains( mapFromGlobal( QCursor::pos() ) ) ) {
		emit longPressed();

		/* Persist 'Shift' key */
		if ( ( mKeyCode == 50 ) or ( mKeyCode == 50 ) ) {
			mPersist = true;
			mChecked = true;
		}
	}

	mPressed = false;

	/* Stop long-press timer */
	longPressTimer->stop();

	repaint();
};

void Button::repeatBackSpace() {

	/* BackSpace key is 22 */
	if ( rect().contains( mapFromGlobal( QCursor::pos() ) ) ) {
		emit sendKey( 3722 );
	}
};

void Button::mousePressEvent( QMouseEvent *mEvent ) {

	mPressed = true;
	longPressTimer->start();

	mEvent->accept();

	repaint();
};

void Button::mouseReleaseEvent( QMouseEvent *mEvent ) {

	/* To enable long-press keys */
	if ( not mPressed )
		return;

	/* Unset the pressed flag */
	mPressed = false;

	/* Disable persistance */
	mPersist = false;

	/* Stop long-press timer */
	longPressTimer->stop();

    if ( mCheckable ) {
		mChecked = not mChecked;
		emit clicked();
	}

    else if ( rect().contains( mEvent->pos() ) ) {
        emit clicked();
        emit sendKey( keyCode() );
    }

    mEvent->accept();

    repaint();
};

void Button::paintEvent( QPaintEvent *pEvent ) {

	if ( not isEnabled() )
		return;

	QPainter *painter = new QPainter( this );
	painter->setRenderHints( QPainter::Antialiasing );

	painter->fillRect( rect(), Qt::transparent );

	painter->save();

	painter->setPen( QColor( 0, 0, 0, 150 ) );
	painter->setBrush( QColor( 0, 0, 0, 50 ) );

	/* If we want to hide key borders */
//	if ( not CKSettings::showKeyBorders() ) {
//		if ( mKeyCode != 65 ) {
//			painter->setPen( Qt::NoPen );
//			painter->setBrush( Qt::NoBrush );
//		}
//	}

	if ( mPressed or mChecked ) {
		QColor bColor = palette().color( QPalette::Text );
		bColor.setAlphaF( 0.3 );
		painter->setBrush( bColor );
	}

	// painter->drawRoundedRect( rect().adjusted( 0, 0, -1, -1 ), 3, 3 );
	painter->drawRect( rect().adjusted( 0, 0, -1, -1 ) );

	/* If we have a persisting button down, underline it */
	if ( mPersist ) {
		QColor bColor( Qt::darkCyan );
		bColor.setAlphaF( 0.5 );
		painter->setPen( QPen( bColor, 2 ) );
		painter->drawRoundedRect( rect().adjusted( 1, 1, -2, -2 ), 3, 3 );
	}

	painter->restore();

	painter->save();

	QFont btnFont( font() );
	btnFont.setPixelSize( height() * 0.5 );
	painter->setFont( btnFont );

	if ( mName.startsWith( ":/resources/" ) ) {
		qreal h = height() * 0.8;
		painter->drawPixmap( QPointF( ( width() - h ) / 2, ( height() - h ) / 2 ), QPixmap( mName ).scaled( h, h, Qt::KeepAspectRatio, Qt::SmoothTransformation ) );
	}

	else {
		QRect rect = QRect( QPoint( 0, 0 ), size() );
		painter->drawText( rect, Qt::AlignCenter, text );
	}

	painter->restore();
	painter->end();

	QWidget::paintEvent( pEvent );
};
